require 'rails_helper'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods
  
  # Authentiicated controller
  describe Api::Dmo::HousingRequestSharesController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    describe 'POST /hermes/api/dmo/housing_request_shares' do
      context 'authenticated user' do
        let!(:admin) { create(:user, roles: ['Administrator']) }
        let!(:housing_request) { create(:active_request, documents_required: 10) }
        let!(:applicant) { housing_request.main_applicant }
        let!(:lessor) { create(:lessor) }

        before do
          authenticated_header(request, admin)
          post :create, data: { attributes: { housing_request_id: housing_request.id, lessor_id: lessor.id } }
        end

        it 'returns status code 201' do
          expect(response.status).to eq(201)
        end

        it 'returns housing_request_share values' do
          expect(json['data']['id']).not_to be_nil
          expect(json['data']['attributes']['code']).not_to be_nil
          expect(json['data']['attributes']['expire_at']).not_to be_nil
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
          expect(json['data']['attributes']['housing_request_id']).to eq(housing_request.id.to_s)
          expect(json['data']['attributes']['public_url']).not_to be_nil
        end

      end
    end

    describe 'GET /hermes/api/dmo/housing_request_shares/find_by_housing_request_id' do
      let!(:lessor_user) { create(:user, auth_roles: ['bailleur']) }
      let!(:superadmin) { create(:user, auth_roles: ['superadministrateur']) }
      let!(:lessor) { create(:lessor, user: lessor_user) }
      let!(:entity) { create(:entity, lessor: lessor, siret: '12345678901234') }
      let!(:applicant) { create(:applicant, main_employer_siret: entity.siret) }
      let!(:housing_offer) { create(:housing_offer, lessor: lessor) }
      let!(:housing_request) { create(:active_request, documents_required: 10, housing_offer_ids: [housing_offer.id], main_applicant: applicant ) }
      let!(:housing_application) { create(:housing_application, housing_offer: housing_offer, housing_request: housing_request, status: 'ANNULEE' ) }

      context 'As a superadmin' do
        before do
          authenticated_header(request, superadmin)
        end

        it 'returns status code 422 when housing_request_id is nil' do
          get :find_by_housing_request_id
          expect(response.status).to eq(422)
        end

        it 'returns status code 404 when housing_request_share dosent exist' do
          get :find_by_housing_request_id, housing_request_id: housing_request.id
          expect(response.status).to eq(404)
        end

        it 'returns status code 200 when housing_request_share exist and is accessible' do
          housing_application.update!(status: 'TRANSMIS BAILLEUR')
          get :find_by_housing_request_id, housing_request_id: housing_request.id
          expect(response.status).to eq(200)
        end
      end

      context 'As a lessor' do
        before do
          housing_application.update!(status: 'TRANSMIS BAILLEUR')
          authenticated_header(request, lessor_user)
          get :find_by_housing_request_id, housing_request_id: housing_request.id
        end

        it 'returns status code 200 when housing_request_share exist and is accessible' do
          expect(response.status).to eq(200)
        end
      end

      context 'As an unauthorized user' do
        before do
          user = FactoryBot.create(:user)
          housing_application.update!(status: 'TRANSMIS BAILLEUR')
          authenticated_header(request, user)
          get :find_by_housing_request_id, housing_request_id: housing_request.id
        end

        it 'returns status code 401 when housing_request_share exist and is accessible' do
          expect(response.status).to eq(401)
        end
      end
    end
  end

  # Public controller
  describe Api::HousingRequestSharesController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:housing_request_share) { create(:housing_request_share, housing_request: FactoryBot.create(:active_request, documents_required: 10)) }
    let!(:housing_request) { housing_request_share.housing_request }
    let!(:applicant) { housing_request.main_applicant }
    let!(:lessor) { housing_request_share.lessor }

    describe 'GET /api/dmo/housing_request_shares/:id' do
      context 'with valid context' do
        it 'returns status code 200' do
          get :show, id: housing_request_share.id, code: housing_request_share.code, format: 'json'
          expect(response.status).to eq(200)
          expect(json['included'].count).to be >= 1
          expect(json['included'].first['type']).to eq('housing_requests')
          expect(json['included'].first['attributes'].keys).to include('request_status')
          expect(json['included'].last['attributes'].keys).to include('required_documents')
          expect(json['included'].last['attributes']['required_documents']).to be_an_instance_of(Array)
          expect(json['included'].last['attributes']['required_documents'][0]['status']).to eq(nil)
          expect(json['included'].last['attributes']['required_documents']).to be_an_instance_of(Array)
        end
      end
      context 'with invalid context' do
        it 'should return 404 error whith invalid id' do
          get :show, id: 123, code: housing_request_share.code, format: 'json'
          expect(response.status).to eq(404)
        end

        it 'should return 404 error whith invalid code' do
          get :show, id: housing_request_share.id, code: 'abcd', format: 'json'
          expect(response.status).to eq(404)
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  # Authenticated controller
  describe Api::Dmo::HousingRequestSharesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_REQUEST_SHARES = 'API_DMO_HOUSING_REQUEST_SHARES'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator'], auth_roles: ['superadministrateur']) }

    context 'has a admin role' do
      path '/api/dmo/housing_request_shares/find_by_housing_request_id' do
        get 'find housing_request_share' do
          tags TAGS_API_DMO_HOUSING_REQUEST_SHARES
          produces 'application/json'
          security [Bearer: {}]
          parameter name: :housing_request_id, in: :query, type: :string, required: true

          response '422', 'housing_request_id invalid' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:housing_request_id) { '' }

            include_context 'with integration test'
            run_test!
          end

          response '404', 'housing_request_share not_found' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:lessor_user) { create(:user, auth_roles: ['bailleur']) }
            let!(:lessor) { create(:lessor, user: lessor_user) }
            let!(:entity) { create(:entity, lessor: lessor, siret: '12345678901234') }
            let!(:applicant) { create(:applicant, main_employer_siret: entity.siret) }
            let!(:housing_offer) { create(:housing_offer, lessor: lessor) }
            let!(:housing_request) { create(:active_request, documents_required: 10, housing_offer_ids: [housing_offer.id], main_applicant: applicant) }
            let!(:housing_request_id) { housing_request.id.to_s }
            let!(:housing_application) { create(:housing_application, housing_offer: housing_offer, housing_request: housing_request, status: 'ANNULEE' ) }

            include_context 'with integration test'
            run_test!
          end

          response '401', 'access_denied' do
            let!(:user) { create(:user) }
            let(:Authorization) { "Bearer #{token_for(user)}" }
            let!(:lessor_user) { create(:user, auth_roles: ['bailleur']) }
            let!(:lessor) { create(:lessor, user: lessor_user) }
            let!(:entity) { create(:entity, lessor: lessor, siret: '12345678901234') }
            let!(:applicant) { create(:applicant, main_employer_siret: entity.siret) }
            let!(:housing_offer) { create(:housing_offer, lessor: lessor) }
            let!(:housing_request) { create(:active_request, documents_required: 10, housing_offer_ids: [housing_offer.id], main_applicant: applicant) }
            let!(:housing_request_id) { housing_request.id.to_s }
            let!(:housing_application) { create(:housing_application, housing_offer: housing_offer, housing_request: housing_request, status: 'ANNULEE' ) }

            before do
              # This action create the housing_request_share
              housing_application.update!(status: 'TRANSMIS BAILLEUR')
            end

            include_context 'with integration test'
            run_test!
          end

          response '200', 'housing_request_share found' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:lessor_user) { create(:user, auth_roles: ['bailleur']) }
            let!(:lessor) { create(:lessor, user: lessor_user) }
            let!(:entity) { create(:entity, lessor: lessor, siret: '12345678901234') }
            let!(:applicant) { create(:applicant, main_employer_siret: entity.siret) }
            let!(:housing_offer) { create(:housing_offer, lessor: lessor) }
            let!(:housing_request) { create(:active_request, documents_required: 10, housing_offer_ids: [housing_offer.id], main_applicant: applicant) }
            let!(:housing_request_id) { housing_request.id.to_s }
            let!(:housing_application) { create(:housing_application, housing_offer: housing_offer, housing_request: housing_request, status: 'TRANSMIS BAILLEUR' ) }

            before do
              # This action create the housing_request_share
              housing_application.update!(status: 'TRANSMIS BAILLEUR')
            end

            include_context 'with integration test'
            run_test!
          end
        end
      end
      path '/api/dmo/housing_request_shares' do
        post 'create HousingRequestShare' do
          tags TAGS_API_DMO_HOUSING_REQUEST_SHARES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:housing_request_id] = { type: :string }
          json_params[:lessor_id] = { type: :string }

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing_request_share created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:housing_request) { create(:active_request, documents_required: 10) }
            let!(:applicant) { housing_request.main_applicant }
            let!(:lessor) { create(:lessor) }
            let(:params) do
              jsonapi_attributes({
                housing_request_id: housing_request.id,
                lessor_id: lessor.id
              })
            end

            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_request_shares'
            end

          end

          response '422', "missing or invalid required fields [:housing_request_id, :lessor_id]" do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:housing_request) { create(:active_request, documents_required: 10) }
            let!(:applicant) { housing_request.main_applicant }
            let!(:lessor) { create(:lessor) }
            let(:params) do
              jsonapi_attributes({
                housing_request_id: nil,
                lessor_id: nil
              })
            end
            run_test!
          end
        end
      end
    end
  end

  # Public controller
  describe Api::HousingRequestSharesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes

    context 'has a admin role' do
      let!(:housing_request_share) { create(:housing_request_share, housing_request: FactoryBot.create(:active_request, documents_required: 10)) }
      path '/api/dmo/housing_request_shares/{id}' do
        get 'get HousingRequestShare' do
          tags TAGS_API_DMO_HOUSING_REQUEST_SHARES
          produces 'application/json'
          consumes 'application/json'

          parameter name: :id, in: :path, type: :string,  required: true
          parameter name: :code, in: :query, type: :string, required: true

          response '200', 'housing_request_share found' do
            let!(:id) { housing_request_share.id }
            let!(:code) { housing_request_share.code }

            include_context 'with integration test'

            after do |_example|
              json_data = JSON.parse(response.body)['data']
              expect(json_data).to be_a Hash
              expect(json_data['id']).to eq(housing_request_share.id.to_s)
            end
          end

          response '404', 'housing_request_share not found or invalid code' do
            let!(:id) { 123 }
            let!(:code) { 'abcde' }

            run_test!
          end

          response '498', 'housing_request_share expired' do
            before do
              housing_request_share.update(expire_at: DateTime.current - 1.day)
            end

            let!(:id) { housing_request_share.id }
            let!(:code) { housing_request_share.code }

            run_test!
          end
        end
      end
    end
  end
end
