require 'rails_helper'

RSpec.describe Hermes::EntityReservation, type: :model do
  let!(:entity_1) do
    create(
      :entity,
      title: 'Entreprise SA',
      siret: '00000000000001',
      id_rex: '1234'
    )
  end

  let!(:entity_2) do
    create(
      :entity,
      title: 'Test SAS',
      siret: '00000000000002',
      id_rex: '2302'
    )
  end

  let!(:entity_3) do
    create(
      :entity,
      title: 'Test SAS2',
      siret: '00000000000003',
      id_rex: '3001'
    )
  end

  context 'with basic setup' do

    let!(:user_entity_1) {create(:user, group_ids: [entity_1.manager_group_id])}
    let!(:user_entity_2) {create(:user, group_ids: [entity_2.group_id])}
    let!(:user_entity_3) {create(:user, group_ids: [entity_3.supervisor_group_id])}

    let!(:housing_offer_1) do
      create(
        :housing_offer,
        offer_status: 'PUBLIE',
      )
    end
  
    let!(:entity_reservation_1) do
      create(
        :entity_reservation,
        entity: entity_1,
        housing_offer: housing_offer_1,
        from:DateTime.now() - 30.days,
        to: DateTime.now() + 30.days,
      )
    end
  
    let!(:entity_reservation_2) do
      create(
        :entity_reservation,
        entity: entity_2,
        housing_offer: housing_offer_1,
        from: DateTime.now() - 30.days,
        to: DateTime.now() + 30.days,
      )
    end

    let!(:entity_reservation_3) do
      create(
        :entity_reservation,
        entity: entity_3,
        housing_offer: housing_offer_1,
        from: DateTime.now() + 20.days,
        to: DateTime.now() + 30.days,
      )
    end
    
    describe 'on creation' do
      it 'attributes are setup properly' do
        expect(entity_reservation_1.reload.is_active).to eq(true)
        expect(entity_reservation_1.reload.last_is_active.to_date).to eq(Date.today)
        expect(housing_offer_1.reload.reserved).to eq(true)
        expect(housing_offer_1.reload.active_reservations).to eq([entity_1.id.to_s, entity_2.id.to_s])
        expect(housing_offer_1.reload.active_reservations_rex_ids).to eq([entity_1.id_rex.to_s, entity_2.id_rex.to_s])
        expect(housing_offer_1.reload.eligibility_query.json_to_h[:$and][0]['entity_id']['$in']).to eq(housing_offer_1.reload.active_reservations)
        expect(Hermes::Status.all().length).to eq(3)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Réservé par Entreprise SA")
        expect(housing_offer_1.reload.entity_status[1][:title]).to eq("Réservé par Test SAS")
        expect(housing_offer_1.readable_by?(user_entity_1)).to eq(true)
        expect(housing_offer_1.readable_by?(user_entity_2)).to eq(true)
      end
    end
    
    describe 'on non active reservation' do     
      it 'on past date' do
        entity_reservation_1.update(to: DateTime.now() - 2.days)
        expect(entity_reservation_1.reload.is_active).to eq(false)
        expect(entity_reservation_1.reload.last_is_active.to_date).to eq(Date.today)
        expect(housing_offer_1.reload.active_reservations.length).to eq(1)
        expect(housing_offer_1.reload.active_reservations_rex_ids.length).to eq(1)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Fin de réservation pour Entreprise SA")
        expect(housing_offer_1.readable_by?(user_entity_1)).to eq(false)
        expect(housing_offer_1.readable_by?(user_entity_2)).to eq(true)
      end
    end
    
    describe 'on update eligibility' do
      it 'attributes are setup properly' do
        Timecop.freeze(DateTime.now() + 31.days)
        entity_reservation_1.save
        expect(entity_reservation_1.reload.is_active).to eq(false)
        expect(housing_offer_1.reload.active_reservations.length).to eq(1)
        expect(housing_offer_1.reload.active_reservations_rex_ids.length).to eq(1)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[0][:title]).to eq("Fin de réservation pour Entreprise SA")
        expect(housing_offer_1.readable_by?(user_entity_1)).to eq(false)
        expect(housing_offer_1.readable_by?(user_entity_2)).to eq(true)
        Timecop.return
      end
    end
    
    describe 'remove reservation' do
      it 'attributes are setup properly' do
        entity_reservation_2.to =  DateTime.now() - 2.days
        entity_reservation_2.save
        expect(entity_reservation_2.reload.is_active).to eq(false)
        expect(housing_offer_1.reload.reserved).to eq(true)
        expect(Hermes::Status.all().length).to eq(4)
        expect(housing_offer_1.reload.entity_status[1][:title]).to eq("Fin de réservation pour Test SAS")
        expect(housing_offer_1.reload.active_reservations).to eq([entity_1.id.to_s])
        expect(housing_offer_1.reload.active_reservations_rex_ids).to eq([entity_1.id_rex.to_s])
        expect(housing_offer_1.readable_by?(user_entity_1)).to eq(true)
        expect(housing_offer_1.readable_by?(user_entity_2)).to eq(false)
      end
    end

    it 'change_entity_id' do
      entity_reservation_2.update(entity: entity_3)
      expect(housing_offer_1.readable_by?(user_entity_2)).to eq(false)
      expect(housing_offer_1.readable_by?(user_entity_3)).to eq(true)
    end

    it 'update_is_active' do 
      entity_reservation_1.set(from: Date.today - 10.days, to: Date.today - 1.days, updated_at: DateTime.now)
      entity_reservation_2.set(from: Date.today + 1.days, to: Date.today + 10.days, updated_at: DateTime.now)
      entity_reservation_3.set(from: Date.today - 10.days , to: Date.today + 2.days , updated_at: DateTime.now)
      Hermes::EntityReservation.update_is_active
      expect(housing_offer_1.reload.active_reservations).to eq([entity_3.id.to_s])
    end 

    it 'on destroy' do
      entity_reservation_1.destroy
      expect(housing_offer_1.reload.active_reservations).to eq([entity_2.id.to_s])
    end 
  end

  describe 'create_new_reserved_offers_notifications' do 
    let!(:user) { create(:user)}
    let!(:app1) { create(:applicant, user: user, main_employer_siret: entity_1.siret, )}
    let!(:hr1) { create(:housing_request, user: user, is_active: true, main_applicant: app1) }
    let!(:offer11) { create(:housing_offer, offer_status: 'PUBLIE', typology: 'T1', insee_code: '75000' )}
    let!(:offer12) { create(:housing_offer, offer_status: 'PUBLIE', typology: 'T1', insee_code: '75000' )}
    let!(:offer13) { create(:housing_offer, offer_status: 'PUBLIE', typology: 'T1', insee_code: '75000' )}
    let!(:offer14) { create(:housing_offer, offer_status: 'PUBLIE', typology: 'T4', insee_code: '75000' )}
    let!(:entity_reservation1) { create(:entity_reservation, entity: entity_1, housing_offer: offer11, from: DateTime.now() - 20.days, to: DateTime.now() + 30.days )}
    let!(:entity_reservation2) { create(:entity_reservation, entity: entity_1, housing_offer: offer12, from: DateTime.now() - 20.days, to: DateTime.now() + 30.days )}

    it 'works' do 
      expect(hr1.reload.entity_id.to_s).to eq(entity_1.id.to_s)
      expect(hr1.reload.eligibles_offers.count).to eq(3)
      expect do
        Hermes::EntityReservation.create_new_reserved_offers_notifications(DateTime.now - 2.days, DateTime.now + 1.minutes)
      end.to change { enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.length }.by 1

      last_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
      expect(last_job[:args][0]["datas"]["offer_ids"]).to eq([offer12.id.to_s, offer11.id.to_s])
    end 
  end 
end